@@ -30,6 +30,7 @@ from lava_results_app.views import (
     suite_csv_stream,
     suite_csv,
     suite_yaml,
+    suite_testcase_count,
     testcase,
     testcase_yaml,
     testjob,
@@ -161,6 +162,8 @@ urlpatterns = [
         suite_csv_stream, name='lava.results.suite_csv_stream'),
     url(r'^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/yaml$',
         suite_yaml, name='lava.results.suite_yaml'),
+    url(r'^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/\+testcase-count$',
+        suite_testcase_count, name='lava.results.suite_testcase_count'),
     url(r'^testcase/(?P<case_id>[0-9]+)$',
         testcase, name='lava.results.testcase'),
     url(r'^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/(?P<case_id>[0-9]+)$',
