@@ -2,7 +2,6 @@
 from __future__ import unicode_literals
 
 from django.db import models, migrations, transaction
-from lava_scheduler_app import utils
 
 
 def forwards_func(apps, schema_editor):
@@ -12,8 +11,8 @@ def forwards_func(apps, schema_editor):
     # Since migrations run only on master node, we assume the current node
     # should be designated as master.
     is_master = True
-    localhost = utils.get_fqdn()
-    ipaddr = utils.get_ip_address()
+    localhost = "example.com"
+    ipaddr = "0.0.0.0"
 
     # NOTE: RPC2_URL formed below is a guess. The administrator should
     #       revisit the correctness of this URL from the administration
