@@ -73,7 +73,7 @@ class TestHealthCheckScheduling(TestCase):
         self.user = User.objects.create(username="user-01")
         self.last_hc03 = TestJob.objects.create(health_check=True, actual_device=self.device03,
                                                 user=self.user, submitter=self.user,
-                                                start_time=timezone.now(),
+                                                start_time=timezone.now(), is_public=True,
                                                 state=TestJob.STATE_FINISHED, health=TestJob.HEALTH_COMPLETE)
         self.device03.last_health_report_job = self.last_hc03
         self.device03.save()
@@ -209,7 +209,7 @@ class TestHealthCheckScheduling(TestCase):
 
         # Create a job that should be scheduled now
         j = TestJob.objects.create(requested_device_type=self.device_type01,
-                                   user=self.user, submitter=self.user,
+                                   user=self.user, submitter=self.user, is_public=True,
                                    definition=_minimal_valid_job(None))
         schedule(DummyLogger())
         self.device01.refresh_from_db()
@@ -221,7 +221,7 @@ class TestHealthCheckScheduling(TestCase):
 
         # Create a job that should be scheduled after the health check
         j = TestJob.objects.create(requested_device_type=self.device_type01,
-                                   user=self.user, submitter=self.user,
+                                   user=self.user, submitter=self.user, is_public=True,
                                    definition=_minimal_valid_job(None))
         self.device03.refresh_from_db()
         self.last_hc03.submit_time = timezone.now() - timedelta(hours=25)
@@ -253,13 +253,13 @@ class TestHealthCheckScheduling(TestCase):
 
         # Create a job that should be scheduled now
         j01 = TestJob.objects.create(requested_device_type=self.device_type01,
-                                     user=self.user, submitter=self.user,
+                                     user=self.user, submitter=self.user, is_public=True,
                                      definition=_minimal_valid_job(None))
         j02 = TestJob.objects.create(requested_device_type=self.device_type01,
-                                     user=self.user, submitter=self.user,
+                                     user=self.user, submitter=self.user, is_public=True,
                                      definition=_minimal_valid_job(None))
         j03 = TestJob.objects.create(requested_device_type=self.device_type01,
-                                     user=self.user, submitter=self.user,
+                                     user=self.user, submitter=self.user, is_public=True,
                                      definition=_minimal_valid_job(None))
 
         schedule(DummyLogger())
@@ -312,9 +312,9 @@ class TestPriorities(TestCase):
         # Disable health checks
         Device.get_health_check = lambda cls: None
         jobs = []
-        for p in [TestJob.LOW, TestJob.MEDIUM, TestJob.HIGH, TestJob.MEDIUM, TestJob.LOW]:
+        for p in [TestJob.LOW, TestJob.MEDIUM, TestJob.HIGH, TestJob.MEDIUM, TestJob.LOW, 40]:
             j = TestJob.objects.create(requested_device_type=self.device_type01,
-                                       user=self.user, submitter=self.user,
+                                       user=self.user, submitter=self.user, is_public=True,
                                        definition=_minimal_valid_job(None), priority=p)
             jobs.append(j)
 
@@ -327,6 +327,7 @@ class TestPriorities(TestCase):
         self._check_job(jobs[2], TestJob.STATE_SCHEDULED, self.device01)
         self._check_job(jobs[3], TestJob.STATE_SUBMITTED)
         self._check_job(jobs[4], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[5], TestJob.STATE_SUBMITTED)
 
         jobs[2].go_state_finished(TestJob.HEALTH_COMPLETE)
         jobs[2].save()
@@ -340,6 +341,7 @@ class TestPriorities(TestCase):
         self._check_job(jobs[2], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[3], TestJob.STATE_SUBMITTED)
         self._check_job(jobs[4], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[5], TestJob.STATE_SUBMITTED)
 
         jobs[1].go_state_finished(TestJob.HEALTH_COMPLETE)
         jobs[1].save()
@@ -353,11 +355,26 @@ class TestPriorities(TestCase):
         self._check_job(jobs[2], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[3], TestJob.STATE_SCHEDULED, self.device01)
         self._check_job(jobs[4], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[5], TestJob.STATE_SUBMITTED)
 
         jobs[3].go_state_finished(TestJob.HEALTH_COMPLETE)
         jobs[3].save()
         self._check_job(jobs[3], TestJob.STATE_FINISHED, self.device01)
 
+        schedule(log)
+        self.device01.refresh_from_db()
+        self.assertEqual(self.device01.state, Device.STATE_RESERVED)
+        self._check_job(jobs[0], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[1], TestJob.STATE_FINISHED, self.device01)
+        self._check_job(jobs[2], TestJob.STATE_FINISHED, self.device01)
+        self._check_job(jobs[3], TestJob.STATE_FINISHED, self.device01)
+        self._check_job(jobs[4], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[5], TestJob.STATE_SCHEDULED, self.device01)
+
+        jobs[5].go_state_finished(TestJob.HEALTH_COMPLETE)
+        jobs[5].save()
+        self._check_job(jobs[5], TestJob.STATE_FINISHED, self.device01)
+
         schedule(log)
         self.device01.refresh_from_db()
         self.assertEqual(self.device01.state, Device.STATE_RESERVED)
@@ -366,6 +383,7 @@ class TestPriorities(TestCase):
         self._check_job(jobs[2], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[3], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[4], TestJob.STATE_SUBMITTED)
+        self._check_job(jobs[5], TestJob.STATE_FINISHED, self.device01)
 
         jobs[0].go_state_finished(TestJob.HEALTH_COMPLETE)
         jobs[0].save()
@@ -379,6 +397,7 @@ class TestPriorities(TestCase):
         self._check_job(jobs[2], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[3], TestJob.STATE_FINISHED, self.device01)
         self._check_job(jobs[4], TestJob.STATE_SCHEDULED, self.device01)
+        self._check_job(jobs[5], TestJob.STATE_FINISHED, self.device01)
 
     def test_low_medium_high_with_hc(self):
         # Enable health checks
@@ -391,7 +410,7 @@ class TestPriorities(TestCase):
         jobs = []
         for p in [TestJob.LOW, TestJob.MEDIUM, TestJob.HIGH, TestJob.MEDIUM, TestJob.LOW]:
             j = TestJob.objects.create(requested_device_type=self.device_type01,
-                                       user=self.user, submitter=self.user,
+                                       user=self.user, submitter=self.user, is_public=True,
                                        definition=_minimal_valid_job(None), priority=p)
             jobs.append(j)
 
