@@ -7,13 +7,13 @@ from lava_scheduler_app.views import (
     device_type_health_history_log,
     mydevices_health_history_log, device_list, device_reports,
     device_restrict_device, device_type_detail, device_dictionary,
-    device_dictionary_plain,
+    device_dictionary_plain, maintenance_devices,
     device_type_reports, edit_worker_desc,
     failure_report, favorite_jobs,
     get_remote_definition, health_job_list, healthcheck, index,
-    job_annotate_failure, job_cancel, job_change_priority, job_complete_log,
+    job_annotate_failure, job_cancel, job_fail, job_change_priority, job_complete_log,
     job_definition, job_definition_plain, job_description_yaml, job_detail,
-    job_list,
+    job_list, job_configuration, job_errors,
     job_log_file_plain, job_log_pipeline_incremental,
     job_pipeline_timing, job_resubmit, job_section_log, job_status,
     job_submit, job_toggle_favorite, lab_health,
@@ -32,6 +32,7 @@ urlpatterns = [
     url(r'^edit_worker_desc', edit_worker_desc, name='lava.scheduler.edit_worker_desc'),
     url(r'^activejobs$', active_jobs, name='lava.scheduler.job.active'),
     url(r'^alljobs$', job_list, name='lava.scheduler.job.list'),
+    url(r'joberrors$', job_errors, name='lava.scheduler.job.errors'),
     url(r'^jobsubmit$', job_submit, name='lava.scheduler.job.submit'),
     url(r'^device_type/(?P<pk>[-_a-zA-Z0-9]+)$', device_type_detail,
         name='lava.scheduler.device_type.detail'),
@@ -66,6 +67,8 @@ urlpatterns = [
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/multinode_definition/plain$',
         multinode_job_definition_plain,
         name='lava.scheduler.job.multinode_definition.plain'),
+    url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/configuration$', job_configuration,
+        name='lava.scheduler.job.configuration'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/log_file/plain$', job_log_file_plain,
         name='lava.scheduler.job.log_file.plain'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/timing$',
@@ -80,6 +83,8 @@ urlpatterns = [
         name='lava.scheduler.job.section_log'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/cancel$', job_cancel,
         name='lava.scheduler.job.cancel'),
+    url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/fail$', job_fail,
+        name='lava.scheduler.job.fail'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/resubmit$', job_resubmit,
         name='lava.scheduler.job.resubmit'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/annotate_failure$',
@@ -116,6 +121,8 @@ urlpatterns = [
         name='lava.scheduler.online_devices'),
     url(r'^alldevices/passinghealthchecks$', passing_health_checks,
         name='lava.scheduler.passing_health_checks'),
+    url(r'^alldevices/maintenance$', maintenance_devices,
+        name='lava.scheduler.maintenance_devices'),
     url(r'^reports/device/(?P<pk>[-_a-zA-Z0-9.]+)', device_reports,
         name='lava.scheduler.device_report'),
     url(r'^reports/device_type/(?P<pk>[-_a-zA-Z0-9]+)', device_type_reports,
