@@ -31,10 +31,9 @@ import yaml
 from collections import OrderedDict
 
 from django.contrib.sites.models import Site
+from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 
-from lava_server.settings.getsettings import Settings
-
 from lava_scheduler_app.schema import SubmissionException
 
 DEFAULT_IRC_SERVER = "irc.freenode.net"
@@ -156,6 +155,7 @@ def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-
         'visibility',
         'notify',
         'metadata',
+        'reboot_to_fastboot',
     ]
     skip = ['role', 'roles']
     scheduling = ['device_type', 'connection', 'host_role', 'context']  # top level values to be preserved
@@ -262,7 +262,8 @@ def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-
             if role not in submission['protocols']['lava-lxc']:
                 continue
             # populate the lava-lxc protocol metadata
-            jobs[role][0]['protocols'].update({'lava-lxc': submission['protocols']['lava-lxc'][role]})
+            for job in jobs[role]:
+                job['protocols'].update({'lava-lxc': submission['protocols']['lava-lxc'][role]})
 
     return jobs
 
@@ -386,15 +387,16 @@ def get_ldap_user_properties(ldap_user):
 
     If given ldap_user does not exist, then raise ldap.NO_SUCH_OBJECT
     """
-    settings = Settings("lava-server")
-    server_uri = settings.get_setting("AUTH_LDAP_SERVER_URI", None)
-    bind_dn = settings.get_setting("AUTH_LDAP_BIND_DN", None)
-    bind_password = settings.get_setting("AUTH_LDAP_BIND_PASSWORD", None)
-    user_dn_template = settings.get_setting("AUTH_LDAP_USER_DN_TEMPLATE", None)
-    user_search = settings.get_setting("AUTH_LDAP_USER_SEARCH", None)
+    server_uri = settings.AUTH_LDAP_SERVER_URI
+    bind_dn = settings.AUTH_LDAP_BIND_DN
+    bind_password = settings.AUTH_LDAP_BIND_PASSWORD
+    user_dn_template = settings.AUTH_LDAP_USER_DN_TEMPLATE
+    user_search = settings.AUTH_LDAP_USER_SEARCH
 
     search_scope = ldap.SCOPE_SUBTREE
-    attributes = ['uid', 'givenName', 'sn', 'mail']
+    # Attributes should be byte strings
+    # (see https://github.com/pyldap/pyldap/issues/68)
+    attributes = [b'uid', b'givenName', b'sn', b'mail']
     search_filter = "cn=*"
 
     if user_dn_template:
