@@ -21,6 +21,7 @@
 import csv
 
 from django.core.management.base import BaseCommand, CommandError, CommandParser
+from django.db import transaction
 
 from lava_scheduler_app.models import Worker
 
@@ -40,12 +41,16 @@ class Command(BaseCommand):
                 super(SubParser, self).__init__(cmd, **kwargs)
 
         sub = parser.add_subparsers(dest="sub_command", help="Sub commands", parser_class=SubParser)
+        sub.required = True
 
         add_parser = sub.add_parser("add", help="Create a worker")
         add_parser.add_argument("hostname", type=str,
                                 help="Hostname of the worker")
         add_parser.add_argument("--description", type=str, default="",
                                 help="Worker description")
+        add_parser.add_argument("--health", type=str, default="ACTIVE",
+                                choices=["ACTIVE", "MAINTENANCE", "RETIRED"],
+                                help="Worker health")
 
         details_parser = sub.add_parser("details", help="Details of a worker")
         details_parser.add_argument("hostname", type=str,
@@ -55,8 +60,8 @@ class Command(BaseCommand):
                                     help="Print the list of attached devices")
 
         list_parser = sub.add_parser("list", help="List the workers")
-        list_parser.add_argument("--all", default=False, action="store_true",
-                                 help="Show all workers (including hidden ones)")
+        list_parser.add_argument("-a", "--all", default=False, action="store_true",
+                                 help="Show all workers (including retired ones)")
         list_parser.add_argument("--csv", dest="csv", default=False,
                                  action="store_true", help="Print as csv")
 
@@ -74,7 +79,7 @@ class Command(BaseCommand):
         """ Forward to the right sub-handler """
         if options["sub_command"] == "add":
             self.handle_add(options["hostname"], options["description"],
-                            options["disabled"])
+                            options["health"])
         elif options["sub_command"] == "details":
             self.handle_details(options["hostname"], options["devices"])
         elif options["sub_command"] == "list":
@@ -83,15 +88,23 @@ class Command(BaseCommand):
             self.handle_update(options["hostname"], options["description"],
                                options["health"])
 
-    def handle_add(self, name, description):
+    def handle_add(self, hostname, description, health_str):
         """ Create a worker """
         try:
             Worker.objects.get(hostname=hostname)
             raise CommandError("Worker already exists with hostname %s" % hostname)
         except Worker.DoesNotExist:
             pass
+
+        if health_str == "ACTIVE":
+            health = Worker.HEALTH_ACTIVE
+        elif health_str == "MAINTENANCE":
+            health = Worker.HEALTH_MAINTENANCE
+        else:
+            health = Worker.HEALTH_RETIRED
         Worker.objects.create(hostname=hostname,
-                              description=description)
+                              description=description,
+                              health=health)
 
     def handle_details(self, hostname, print_devices):
         try:
@@ -118,7 +131,7 @@ class Command(BaseCommand):
             workers = workers.exclude(health=Worker.HEALTH_RETIRED)
 
         if format_as_csv:
-            fields = ["hostname", "description", "master", "hidden", "devices"]
+            fields = ["hostname", "description", "master", "state", "health", "devices"]
             writer = csv.DictWriter(self.stdout, fieldnames=fields)
             writer.writeheader()
             for worker in workers:
@@ -135,18 +148,19 @@ class Command(BaseCommand):
 
     def handle_update(self, hostname, description, health):
         """ Update worker properties """
-        try:
-            worker = Worker.objects.get(hostname=hostname)
-        except Worker.DoesNotExist:
-            raise CommandError("No worker exists with hostname %s" % hostname)
+        with transaction.atomic():
+            try:
+                worker = Worker.objects.select_for_update().get(hostname=hostname)
+            except Worker.DoesNotExist:
+                raise CommandError("No worker exists with hostname %s" % hostname)
 
-        if description is not None:
-            worker.description = description
-        if health is not None:
-            if health == "ACTIVE":
-                worker.go_health_active()
-            elif health == "MAINTENANCE":
-                worker.go_health_maintenance()
-            else:
-                worker.go_health_retired()
-        worker.save()
+            if description is not None:
+                worker.description = description
+            if health is not None:
+                if health == "ACTIVE":
+                    worker.go_health_active()
+                elif health == "MAINTENANCE":
+                    worker.go_health_maintenance()
+                else:
+                    worker.go_health_retired()
+            worker.save()
