@@ -19,6 +19,7 @@
 import os
 
 from django.core.wsgi import get_wsgi_application
+from django.db.backends.signals import connection_created
 
 
 # Set the environment variables for Django
@@ -26,5 +27,19 @@ os.environ.setdefault("DJANGO_SETTINGS_MODULE", "lava_server.settings.distro")
 os.environ.setdefault("DJANGO_DEBIAN_SETTINGS_TEMPLATE",
                       "/etc/lava-server/{filename}.conf")
 
+
+# Set the postgresql connection timeout
+# The timeout is only used when running through wsgi.  This way, command line
+# commands are not affected by the timeout.
+def setup_postgres(connection, **kwargs):
+    if connection.vendor != 'postgresql':
+        return
+
+    with connection.cursor() as cursor:
+        cursor.execute("SET statement_timeout TO 30000")
+
+
+connection_created.connect(setup_postgres, dispatch_uid="setup_postgres")
+
 # Create the application
 application = get_wsgi_application()
