@@ -21,7 +21,7 @@ Unit tests for Linaro Django XML-RPC Application
 """
 import re
 import logging
-import xmlrpclib
+import sys
 
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
@@ -38,6 +38,13 @@ from linaro_django_xmlrpc.models import (
     xml_rpc_signature,
 )
 
+if sys.version_info[0] == 2:
+    # Python 2.x
+    import xmlrpclib
+elif sys.version_info[0] == 3:
+    # For Python 3.0 and later
+    import xmlrpc.client as xmlrpclib
+
 
 class MockUser(object):
     """
@@ -173,7 +180,7 @@ class MapperTests(TestCase):
         context = CallContext(None, self.mapper, None)
         retval = self.mapper.lookup('API.foo', context)
         # bound method seems to have im_self attribute pointing back to self
-        self.assertIs(retval.im_self._context, context)
+        self.assertIs(retval.__self__._context, context)
 
     def test_list_methods_without_methods(self):
         class TestAPI(ExposedAPI):
